#include "classicgame.h"
#include "classictimeline.h"
#include "classicmapcreator.h"
#include "classicnote.h"

ClassicGame::ClassicGame(std::unique_ptr<ClassicGraphicsManager>&& manager) :
    _timeline(std::make_unique<ClassicTimeline>()),
    _graphics_manager(std::move(manager))
{
    _slap_buffer.loadFromFile("very-final-slap.wav");
    _slap.setBuffer(_slap_buffer);
    _slap.setVolume(50);

    _keys_to_buttons =
    {
        {sf::Keyboard::Up, Type::UP},      // Load from settings
        {sf::Keyboard::Right, Type::RIGHT},
        {sf::Keyboard::Down, Type::DOWN},
        {sf::Keyboard::Left, Type::LEFT},

        {sf::Keyboard::W, Type::UP},
        {sf::Keyboard::D, Type::RIGHT},
        {sf::Keyboard::S, Type::DOWN},
        {sf::Keyboard::A, Type::LEFT},

        {sf::Keyboard::E, Type::SLIDER_RIGHT},
        {sf::Keyboard::Q, Type::SLIDER_LEFT}
    };

    _buttons_to_pressed_actions=
    {
        {Type::UP, Action::PRESS_UP},
        {Type::RIGHT, Action::PRESS_RIGHT},
        {Type::DOWN, Action::PRESS_DOWN},
        {Type::LEFT, Action::PRESS_LEFT},

        {Type::SLIDER_RIGHT, Action::PRESS_SLIDER_RIGHT},
        {Type::SLIDER_LEFT, Action::PRESS_SLIDER_LEFT}
    };

    _buttons_to_released_actions=
    {
        {Type::UP, Action::RELEASE_UP},
        {Type::RIGHT, Action::RELEASE_RIGHT},
        {Type::DOWN, Action::RELEASE_DOWN},
        {Type::LEFT, Action::RELEASE_LEFT},

        {Type::SLIDER_RIGHT, Action::RELEASE_SLIDER_RIGHT},
        {Type::SLIDER_LEFT, Action::RELEASE_SLIDER_LEFT}
    };
}

ClassicGame::~ClassicGame()
{}

void ClassicGame::run()
{
    ClassicMapCreator creator(_graphics_manager);
    auto beatmap = creator.createBeatmap("aa");
    _timeline->run(std::move(beatmap.notes), beatmap.visibility_offset);
}

void ClassicGame::input(PlayerInput&& inputdata)
{
    inputdata.timestamp = _timeline->currentMusicOffset();

    switch (inputdata.event.type)
    {
    default:
        return;
        break;        
    case sf::Event::KeyPressed:
    case sf::Event::KeyReleased:
        {
            auto note = _timeline->getActiveNote();

            if (!_timeline->isExpired(note))
            {
                (*note)->input(std::move(inputdata));
                _slap.play();
            }
        }
        break;
    }
}

void ClassicGame::update()
{
    _timeline->update();
    _timeline->fetchVisibleNotes();
}

void ClassicGame::draw() const
{
    _timeline->drawVisibleNotes();
}
