#include "classicsprite.h"
#include <SFML/Graphics/RenderTarget.hpp>

ClassicSprite::ClassicSprite(const sf::RectangleShape& shape, const sf::Font& font) :
    _prototype(shape),
    _shape(shape),
    _trail(shape),
    _font(font)
{
    _grade_text.setFont(_font);
}

void ClassicSprite::draw(sf::RenderTarget& target, sf::RenderStates states) const
{
    target.draw(_shape, states);
    target.draw(_trail, states);
    target.draw(_grade_text, states);
}

void ClassicSprite::reset()
{
    _shape.setPosition(0, 0);
    _trail.setPosition(0, 0);
    _grade_text.setPosition(0, 0);
    _grade_text.setFillColor(sf::Color(255, 255, 255, 0));

    _shape = _prototype;
    _trail = _prototype;
    _trail_fade = false;
}

void ClassicSprite::setCoordinates(float x, float y, float trail_x, float trail_y) noexcept
{
    _shape.setPosition(x, y);
    _trail.setPosition(trail_x, trail_y);
    _grade_text.setPosition(x + _shape.getSize().x/2, y + 10);
}

void ClassicSprite::update(float trail_x, float trail_y) noexcept
{
    _trail.setPosition(trail_x, trail_y);

    if (_trail_fade)
    {
        auto fill_color = _trail.getFillColor();

        if (fill_color.a == 0)
            return;

        auto new_alpha = fill_color.a - 35;
        fill_color.a = new_alpha < 0 ? 0 : new_alpha;

        _trail.setFillColor(fill_color);
    }
}

void ClassicSprite::update() noexcept
{
    fade();
}

void ClassicSprite::pulse()
{
    _grade_text.setFillColor(sf::Color(255, 255, 255, 255));
    _shape.setFillColor(sf::Color(140, 140, 140));
    _trail.setPosition(0, 0);
    _trail.setFillColor(sf::Color(0, 0, 0, 0));
}

void ClassicSprite::fade()
{
    auto fill_color = _grade_text.getFillColor();

    if (fill_color.a == 0)
        return;

    auto new_alpha = fill_color.a - 15;
    fill_color.a = new_alpha < 0 ? 0 : new_alpha;

    _grade_text.setFillColor(fill_color);

    fill_color = _shape.getFillColor();

    if (fill_color.a == 0)
        return;

    new_alpha = fill_color.a - 15;
    fill_color.a = new_alpha < 0 ? 0 : new_alpha;

    _shape.setFillColor(fill_color);
}

void ClassicSprite::trailFade()
{
    _trail_fade = true;
}

bool ClassicSprite::isDead() const
{
    return _grade_text.getFillColor().a == 0
        ||      _shape.getFillColor().a == 0;
}
